#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

#define PIN_LED 2
#define PIN_BUTTON 3

// Set the LCD address to 0x3f or 0x27 for a 20 chars / 4 line display
LiquidCrystal_I2C lcd(0x27, 20, 4);

// global variables

/* setup() --------------------------------------------------------------*/
void setup() {
  pinMode(PIN_LED, OUTPUT);                   // set Ports
  pinMode(PIN_BUTTON, INPUT_PULLUP);
  digitalWrite(PIN_LED, LOW);                 // switch LED off
  
  lcd.init();                                 // initialize the LCD
  lcd.backlight();                            // activate backlight
  lcd.setCursor(0, 0);                        // set cursor and show title
  lcd.print("Reaktionszeit-Tester");
}

/* loop() ---------------------------------------------------------------*/
void loop()
{
  int reac_time = 0;                          // reaction time in msec
  int wait_time = random(17000);              // wait time in msec, initialize with random number
  boolean too_early_flag = false;             // flag for error (key pressed too early)
  int i;

  // preparations
  while (digitalRead(PIN_BUTTON) == LOW);     // wait for button to be released
  lcd.setCursor(0, 2);                        // set cursor and show title
  lcd.print("Fertig?          ");
  while (digitalRead(PIN_BUTTON) == HIGH);    // wait for button

  // run wait time between 3 and 20 secs
  lcd.setCursor(0, 2);                        // set cursor and show title
  lcd.print("Achtung!");
  delay(3000);                                // wait 3 seconds
  for (i = 0; i < wait_time; ++i) {           // wait for a random time (<= 17 secs)
    if (digitalRead(PIN_BUTTON) == LOW) {     // check whether button was pressed
      too_early_flag = true;                    
      break;    
    };
    delay(1);                                 
  };

  lcd.setCursor(0, 2);                        // set cursor 
  if (too_early_flag == false) {

    // run test
    lcd.print("Jetzt ...");
    digitalWrite(PIN_LED, HIGH);              // switch LED on
    for (reac_time = 0; reac_time < 10000; ++reac_time) {
      if (digitalRead(PIN_BUTTON) == LOW) break;
      delay(1);
    };
    digitalWrite(PIN_LED, LOW);               // switch LED on

    // show result
    lcd.setCursor(0, 2);                      // set cursor 
    if (reac_time == 10000) {
      lcd.print("Zu spaet ...");
    } else {
      lcd.print("Ergebnis: ");
      lcd.print(reac_time);      
    };
    
  } else {
    // show error message
    lcd.print("Fehler, zu frueh!");           // print error message
  };
  while (digitalRead(PIN_BUTTON) == LOW);     // wait for button to be released
  delay(1000);

  // done, wait for key pressed to start the next round
  while (digitalRead(PIN_BUTTON) == HIGH);    // wait for button    
}

